#
# MailStats::Article
#
#	Based on News::Scan::Article by Greg Bacon
#
#	Version 1.0 - 12/97 - Alan Jackson


package MailStats::Article;

use strict;
use vars qw( $VERSION );

use Carp;
use Date::Parse;
use Mail::Address;

require MailStats::Group;

$VERSION = sprintf "%d.%02d", q$Revision: 1.0 $ =~ m#(\d+)\.(\d+)#;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    my $group;
    my $file;

    croak "usage: ${class}->new(GROUP-OBJ, FILENAME)" unless @_ == 2;

    $group = shift;
    $file  = shift;

    return undef unless $file =~ /^\d+$/;

    bless $self, $class;
    $self->group($group);
    $self->source($file);

    if (defined $self->measure) {
        return $self;
    }
    else {
        return undef;
    }
}

sub group {
    my $self = shift;

    if (@_) {
        $self->{group} = shift;
    }
    else {
        return $self->{group};
    }
}

sub source {
    my $self = shift;

    if (@_) {
        $self->{source} = shift;
    }
    else {
        return $self->{source};
    }
}

sub size {
    my $self = shift;

    if (@_) {
        $self->{size} = shift;
    }
    else {
        return $self->{size};
    }
}

sub header_size {
    my $self = shift;

    if (@_) {
        $self->{header_size} = shift;
    }
    else {
        return $self->{header_size};
    }
}

sub header_lines {
    my $self = shift;

    if (@_) {
        $self->{header_lines} = shift;
    }
    else {
        return $self->{header_lines};
    }
}

sub body_size {
    my $self = shift;

    if (@_) {
        $self->{body_size} = shift;
    }
    else {
        return $self->{body_size};
    }
}

sub body_lines {
    my $self = shift;

    if (@_) {
        $self->{body_lines} = shift;
    }
    else {
        return $self->{body_lines};
    }
}

sub body_words {
    my $self = shift;

    if (@_) {
        $self->{body_words} = shift;
    }
    else {
        return $self->{body_words};
    }
}

sub orig_size {
    my $self = shift;

    if (@_) {
        $self->{orig_size} = shift;
    }
    else {
        return $self->{orig_size};
    }
}

sub orig_lines {
    my $self = shift;

    if (@_) {
        $self->{orig_lines} = shift;
    }
    else {
        return $self->{orig_lines};
    }
}

sub orig_words {
    my $self = shift;

    if (@_) {
        $self->{orig_words} = shift;
    }
    else {
        return $self->{orig_words};
    }
}

sub measure {
    my $self = shift;
    my $file = $self->group->spool . $self->source;
    my $art;

    ## just how big are we?
    $self->size(-s $file);

#print "---> $file\n";

    open ART, $file or croak "$0: failed open [$file]: $!";
    $art = join '', <ART>;
    close ART;

    ################################
    # deal with headers
    ################################

    my $header;
    my %header;

    # CHOP!! right at the neck! :-)
    unless ($art =~ s/^(.*?\n)\n//s) {
        warn "$0: no header in bogus article `$file'!";
        return undef;
    }
    $header = $1;

	#	Look for the X-stats field - have I already captured this message?
	my $testflag = $self->group->testflag;
	if ($testflag ne "test") {
		 if (index($header,"X-stats: " ) > -1) {
			 return undef;
		 }
	}

    $self->header_size(length($header));
    $self->header_lines($header =~ tr/\n/\n/);

    ## fix continuation -- see perlfunc
    $header =~ s/\n\s+/ /g;
    %header = (dummy => split /^(.*?):\s*/m, $header);
    chomp %header;  ## an abuse?

    # remove empty headers
    for (keys %header) {
        delete $header{$_} unless $header{$_} =~ /\S/;
    }

    ## make sure all goes well with the headers
    unless ($self->_process_header(%header)) {

	 	print ">> header failer $file\n";
        return undef;
    }

	 ## update header to add in X-stats: flag

	if ($testflag ne "test") {
		 $header =~ s/Date:/X-stats: MailStats\nDate:/;
	 }

    open (ART,">$file") or croak "$0: failed open [$file]: $!";
	 	print ART $header,"\n",$art;
    close ART;
	 

    ################################
    # deal with the body 
    ################################
    my $body;

    $body = $art;         
    $art  = '';

    $self->body_size(length($body));
    $self->body_lines($body =~ tr/\n/\n/);
	 my $wordcount = ($body =~ s/([^\s\n]+)/$1/g);
	 $self->body_words($wordcount);

    my $quote_re = $self->group->quote_re;
    my @lines;

    for (split /\n/, $body) {
		  if ($header{From} =~ /aol\.com/i) { # aol has special quoting
		  		if (/^\s*<{2}/../\s*>{2}\s*$/) {
					s/^/> /;
				}
		  }
		  push @lines, "$_\n" unless /$quote_re/;
    }
	 
    $self->orig_size(length(join '', @lines));
    $self->orig_lines(scalar @lines);
    my $orig = join '', @lines;
	 $wordcount = ($orig =~ s/([^\s\n]+)/$1/g);
	 $self->orig_words($wordcount);

    1;
}

sub date {
    my $self = shift;

    if (@_) {
        my $date;
        $date = str2time shift;
        $self->{date} = $date;

    }
    else {
        return $self->{date};
    }

    1;
}

sub author {
    my $self = shift;

    if (@_) {
        local $::from    = shift;
        local $::warning = undef;
        local $SIG{'__WARN__'} =
            sub {
                $::warning = "$0: address `$::from' produced:\n$_[0]\n";
                # $::from = 'ADDRONLY';
                $::from = 'REJECTME';
            };
        my $add;

        eval { ($add) = Mail::Address->parse($::from) };
        return undef if $@;  ## we don't like it if MA->parse don't

        ## bummer for the author if this is the first we come across
        if ($::from eq 'REJECTME') {
            warn $::warning;
            # $add->comment('');
            # $add->phrase('');
            return undef;
        }

        ## can we say shattered encapsulation?
        if (exists $self->group->aliases->{lc $add->[1]}) {
            $add->[1] = ${$self->group->aliases}{lc $add->[1]};
        }
        $self->{author} = $add;
    }
    else {
        return $self->{author};
    }

    1;
}

sub subject {
    my $self = shift;
	 my $listname = $self->group->listname;

    if (@_) {
	 	if ($listname) {
        ($self->{subject} = shift) =~ s/(Re:\s*|fw:\s*|fwd:\s*||^-reply)|$listname//ig;
		}
		else {
        ($self->{subject} = shift) =~ s/^Re:\s*|^fw:\s*|fwd:\s*||^-reply//ig;
	   }
    }
    else {
        return $self->{subject};
    }

    1;
}

sub message_id {
    my $self = shift;

    if (@_) {
        $self->{message_id} = shift;
    }
    else {
        return $self->{message_id};
    }
}

sub _process_header {
    my $self   = shift;
    my %header = @_;
    my $file   = $self->source;

    ################################
    # Date:
    ################################
    unless (exists $header{Date}) {
        warn "$0: no Date: header in bogus article `$file'";
        return undef;
    }
    else {
        return undef unless $self->date($header{Date});
    }

    ################################
    # From:
    ################################
    unless (exists $header{From}) {
        warn "$0: no From: header in bogus article `$file'";
        return undef;
    }
    else {
        ## do not prefer Reply-To: to From:
        my $reply = exists $header{'From'}
                      ?
                    $header{From}
                      :
                    $header{'Reply-To'};

        return undef unless $self->author($reply);
    }

    ################################
    # Subject:
    ################################
    unless (exists $header{Subject}) {
        warn "$0: no Subject: header in bogus article `$file'";
        return undef;
    }
    else {
        return undef unless $self->subject($header{Subject});
    }

    ################################
    # Message-ID:
    ################################
    if (exists $header{'Message-ID'}) {
        $self->message_id($header{'Message-ID'});
    }

    1;
}

1;
