#
# MailStats::Group
#
# $Id: Group.pm,v 1.1 1997/02/19 02:45:34 gbacon Exp gbacon $
#		Based on Greg Bacon's Group.pm for newsgroups
#		Alan Jackson 12/97

package MailStats::Group;

use strict;
use vars qw( $VERSION @ISA );

use Carp;
use POSIX;
require MailStats::Article;
require MailStats::Poster;
require MailStats::Thread;
require MailStats::Stats;

@ISA     = qw( MailStats::Stats );
$VERSION = sprintf "%d.%02d", q$Revision: 1.0 $ =~ m#(\d+)\.(\d+)#;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {
        posters    => {},
        threads    => {},
        aliases    => {},
    };

    bless $self, $class;

    if (@_) {
        if (@_ < 1 && @_ > 3 ) {
            croak "usage: ${class}->new([DIRNAME],testflag,listname)";
        }
        else {
            $self->init(@_);
        }
    }

    $self;
}


sub init {
    my $self  = shift;
    my $class = ref($self);

    if (@_ < 1 || @_ > 3) {
        croak "usage: ${class}->init(DIRNAME)";
    }

    my $dir    = shift;
	 my $testflag = "test"; # 'test' or 'run'
	 if (@_) {$testflag = shift};
	 my $listname ;
	 if (@_) {$listname = shift; $self->listname($listname);}

	 # print "$dir : $testflag : $listname\n";

    my $quote_re = '^\s{0,3}(?:>|:|\S+>|\+\+)';

    $self->testflag($testflag);
    $self->spool($dir);
    $self->quote_re($quote_re);

    $self->articles(0);
    $self->volume(0);

    $self->header_volume(0);
    $self->header_lines(0);

    $self->body_volume(0);
    $self->body_lines(0);
    $self->body_words(0);

    $self->orig_volume(0);
    $self->orig_lines(0);
    $self->orig_words(0);

}

sub testflag {
    my $self = shift;

    if (@_) {
        $self->{testflag} = shift;
    }
    else {
        return $self->{testflag};
    }
}

sub listname {
    my $self = shift;

    if (@_) {
        $self->{listname} = shift;
    }
    else {
        return $self->{listname};
    }
}

sub quote_re {
    my $self = shift;

    if (@_) {
        $self->{quote_re} = shift;
    }
    else {
        return $self->{quote_re};
    }
}

sub group {
    my $self = shift;

    if (@_) {
        my $group = shift;

        $self->{group} = $group;
        return;
    }
    else {
        return $self->{group};
    }
}

sub spool {
    my $self = shift;

    if (@_) {
        my $spool = shift;

        $spool .= '/' unless $spool =~ m!/$!;

        $self->{spool} = $spool;
        return;
    }
    else {
        return $self->{spool};
    }
}

sub volume {
    my $self = shift;

    if (@_) {
        $self->{volume} = shift;
        return;
    }
    else {
        return $self->{volume};
    }
}

sub scan {
    my $self = shift;
    my $dir  = $self->spool;
    my $article;
    my $file;
    my @spin = qw( - \\ | / - );
    my $i;

    opendir DIR, $dir or die "$0: failed opendir [$dir]: $!\n";

    ARTICLE:
    while (defined($file = readdir(DIR))) {
	 	next if $file !~ /^\d+$/; # only fully numeric filenames
        $i++;
        print "\rProcessed $i articles ", $spin[$i%@spin];
		  #print ">> $dir / $file\n";

        $article = new MailStats::Article $self, $file;
        if (defined $article ) {
            $self->add_article($article) if defined $article;
        }
    }
}


sub add_article {
    my $self    = shift;
    my $article = shift;

    $self->articles($self->articles + 1);
    $self->volume($self->volume + $article->size);

    $self->header_volume($self->header_volume + $article->header_size);
    $self->header_lines($self->header_lines + $article->header_lines);

    $self->body_volume($self->body_volume + $article->body_size);
    $self->body_lines($self->body_lines + $article->body_lines);
    $self->body_words($self->body_words + $article->body_words);

    $self->orig_volume($self->orig_volume + $article->orig_size);
    $self->orig_lines($self->orig_lines + $article->orig_lines);
    $self->orig_words($self->orig_words + $article->orig_words);

    $self->add_poster($article);
    $self->add_to_thread($article);
}

sub articles {
    my $self = shift;

    if (@_) {
        $self->{articles} = shift;
    }
    else {
        return $self->{articles};
    }
}

sub header_volume {
    my $self = shift;

    if (@_) {
        $self->{header_volume} = shift;
    }
    else {
        return $self->{header_volume};
    }
}

sub header_lines {
    my $self = shift;

    if (@_) {
        $self->{header_lines} = shift;
    }
    else {
        return $self->{header_lines};
    }
}

sub body_volume {
    my $self = shift;

    if (@_) {
        $self->{body_volume} = shift;
    }
    else {
        return $self->{body_volume};
    }
}

sub body_lines {
    my $self = shift;

    if (@_) {
        $self->{body_lines} = shift;
    }
    else {
        return $self->{body_lines};
    }
}

sub body_words {
    my $self = shift;

    if (@_) {
        $self->{body_words} = shift;
    }
    else {
        return $self->{body_words};
    }
}

sub orig_volume {
    my $self = shift;

    if (@_) {
        $self->{orig_volume} = shift;
    }
    else {
        return $self->{orig_volume};
    }
}

sub orig_lines {
    my $self = shift;

    if (@_) {
        $self->{orig_lines} = shift;
    }
    else {
        return $self->{orig_lines};
    }
}

sub orig_words {
    my $self = shift;

    if (@_) {
        $self->{orig_words} = shift;
    }
    else {
        return $self->{orig_words};
    }
}

sub find_poster {
    my $self = shift;
    my $add  = lc shift;

    return exists $self->{posters}{$add}
                    ?
                  $self->{posters}{$add}
                    :
                  undef;
}

sub create_poster {
    my $self = shift;
    my $art  = shift;
    my $poster;
    
    $poster = new MailStats::Poster $art;
    $self->{posters}{lc $art->author->address} = $poster;

    $poster;
}

sub add_poster {
    my $self = shift;
    my $art  = shift;
    my $poster;

    $poster = $self->find_poster($art->author->address);
    unless ($poster) {
        $poster = $self->create_poster($art);
        return;
    }

    $poster->volume($poster->volume + $art->size);
    $poster->articles($poster->articles + 1);
    $poster->header_volume($poster->header_volume + $art->header_size);
    $poster->body_volume($poster->body_volume + $art->body_size);
    $poster->body_words($poster->body_words + $art->body_words);
    $poster->orig_volume($poster->orig_volume + $art->orig_size);
    $poster->orig_words($poster->orig_words + $art->orig_words);
}

sub find_thread {
    my $self = shift;
    my $subj = shift;

    return exists $self->{threads}{$subj}
                    ?
                  $self->{threads}{$subj}
                    :
                  undef;
}

sub create_thread {
    my $self = shift;
    my $art  = shift;
    my $thread;

    $thread = new MailStats::Thread $art;
    $self->{threads}{$thread->subject} = $thread;

    $thread;
}

sub add_to_thread {
    my $self = shift;
    my $art  = shift;
    my $thread;

    $thread = $self->find_thread($art->subject);
    unless ($thread) {
        $thread = $self->create_thread($art);
        return;
    }

    $thread->volume($thread->volume + $art->size);
    $thread->articles($thread->articles + 1);
    $thread->header_volume($thread->header_volume + $art->header_size);
    $thread->body_volume($thread->body_volume + $art->body_size);
    $thread->body_words($thread->body_words + $art->body_words);
    $thread->orig_volume($thread->orig_volume + $art->orig_size);
    $thread->orig_words($thread->orig_words + $art->orig_words);
}

sub aliases {
    my $self = shift;
    local($_);

    if (@_) {
        my %aliases = @_;
        for (keys %aliases) {
            $self->{'aliases'}->{$_} = $aliases{$_};
        }
    }
    else {
        return $self->{'aliases'};
    }
}

1;
