#
# News::Scan::Stats
#
# $Id: Stats.pm,v 1.1 1997/02/19 02:45:34 gbacon Exp gbacon $
#		Alan Jackson 12/97

package MailStats::Stats;

use strict;
use vars qw( $VERSION %thr_cnt);

use Carp;
use POSIX;

$VERSION = sprintf "%d.%02d", q$Revision: 1.0 $ =~ m#(\d+)\.(\d+)#;

sub _places {
    my $places = shift;
    my $q      = shift;

    sprintf "%.${places}f", $q;
}

sub _commify {
    my $n = shift;

    1 while $n =~ s/(\d)(\d\d\d)(?!\d)/$1,$2/g;

    $n;
}

sub stats {
    my $self = shift;
	 my $month = shift;

    my $quote_re = $self->quote_re;
    my $posters  = scalar keys %{$self->{posters}};
    my $posts    = $self->articles;
    my $threads  = scalar keys %{$self->{threads}};
    my $volume   = _places(1, $self->volume / 1024);

    my $headers = _places(1, $self->header_volume / 1024);
    my $h_lines = _commify $self->header_lines;

    my $body    = _places(1, $self->body_volume   / 1024);
    my $b_lines = _commify $self->body_lines;
    my $b_words = _commify $self->body_words;

    my $orig    = _places(1, $self->orig_volume   / 1024);
    my $o_lines = _commify $self->orig_lines;
    my $o_words = _commify $self->orig_words;

    my $avg_posts  = _places(1, $posts               / $posters);
    my $avg_thread = _places(1, $posts               / $threads);
    my $avg_vol    = _places(1, $self->volume        / $self->articles);

    my $avg_header = _places(1, $self->header_volume / $posts);
    my $avg_hlines = _places(1, $self->header_lines  / $posts);

    my $avg_body   = _places(1, $self->body_volume   / $posts);
    my $avg_blines = _places(1, $self->body_lines    / $posts);
    my $avg_bwords = _places(1, $self->body_words    / $posts);

    my $avg_orig   = _places(1, $self->orig_volume   / $posts);
    my $avg_olines = _places(1, $self->orig_lines    / $posts);
    my $avg_owords = _places(1, $self->orig_words    / $posts);

    my $ocr        = 100* _places(4, $self->orig_volume   / $self->body_volume);
    my $ocr_words  = 100* _places(4, $self->orig_words   / $self->body_words);

    print <<EOF;
Following is a summary of articles for the month of $month.

Notes
=====

    - A line in the body of a post is considered to be original if it
      does *not* match the regular expression /$quote_re/.
    - Original Content Rating is the ratio of the original content volume
      to the total body volume. Large numbers are good.
    - Please send all comments to Alan Jackson <ajackson\@icct.net>.
    - The software that produced this report was modified from Greg
      Bacon's exellent package developed for the comp.lang.perl.misc
      newsgroup.

Totals
======

Total number of posters:  $posters
Total number of articles: $posts 
Total number of threads:  $threads
Total volume generated:   $volume kb
    - headers:    $headers kb ($h_lines lines)
    - bodies:     $body kb ($b_lines lines, $b_words words)
    - original:   $orig kb ($o_lines lines, $o_words words)
Original Content Rating (non-quoted/total): 
$ocr% (kbytes), $ocr_words% (words)

Averages
========

Number of posts per poster: $avg_posts
Number of posts per thread: $avg_thread
Message size: $avg_vol bytes
    - header:     $avg_header bytes ($avg_hlines lines)
    - body:       $avg_body bytes ($avg_blines lines $avg_bwords words)
    - original:   $avg_orig bytes ($avg_olines lines $avg_owords words)

Top 10 Posters by Number of Posts
=================================

         (kb)   (kb)  (kb)  (kb)
Posts  Volume (  hdr/ body/ orig)  Address
-----  --------------------------  -------

EOF

    ## my, we do love The Black Transform :-)
    my @top =
             (
              map  { $_->[0] }
              sort { $b->[1] <=> $a->[1] }
              map  { [ $_, $self->{posters}{$_}->articles ] }
              keys %{$self->{posters}}
             )[0 .. 9];

    for (@top) {
        my $poster = $self->{posters}{$_};
        my $vol = sprintf "%5.1f (%5.1f/%5.1f/%5.1f)",
                          $poster->volume / 1024,
                          $poster->header_volume / 1024,
                          $poster->body_volume / 1024,
                          $poster->orig_volume / 1024;

        printf "%5d  %26s  %s\n", $poster->articles, $vol,
               $poster->attrib;
    }

    print <<EOF;

Top 10 Posters by Volume
========================

  (kb)   (kb)  (kb)  (kb)
Volume (  hdr/ body/ orig)  Posts  Address
--------------------------  -----  -------

EOF

    @top =
          (
           map  { $_->[0] }
           sort { $b->[1] <=> $a->[1] }
           map  { [ $_, $self->{posters}{$_}->volume ] }
           keys %{$self->{posters}}
          )[0 .. 9];

    for (@top) {
        my $poster = $self->{posters}{$_};
        my $vol = sprintf "%5.1f (%5.1f/%5.1f/%5.1f)",
                          $poster->volume / 1024,
                          $poster->header_volume / 1024,
                          $poster->body_volume / 1024,
                          $poster->orig_volume / 1024;

        printf "%26s  %5d  %s\n", $vol, $poster->articles,
               $poster->attrib;
    }

    print <<EOF;

Top 10 Posters by OCR (minimum of three posts)
==============================================

          (kb)    (kb)
OCR       orig /  body  Posts  Address
------  --------------  -----  -------

EOF

    @top =
          (
           sort { $b->[1] <=> $a->[1] }
           map  { [ $_, ($self->{posters}{$_}->orig_volume
                         / $self->{posters}{$_}->body_volume) ] }
           grep { $self->{posters}{$_}->articles >= 3 }
           keys %{$self->{posters}}
          )[0 .. 9];

    for (@top) {
        my $poster = $self->{posters}{$_->[0]};

        printf "%.4f   %5.1f / %5.1f  %5d  %s\n",
               $_->[1], $poster->orig_volume / 1024,
               $poster->body_volume / 1024,
               $poster->articles, $poster->attrib;
    }

print <<EOF;

Bottom 10 Posters by OCR (minimum of three posts)
=================================================

          (kb)    (kb)
OCR       orig /  body  Posts  Address
------  --------------  -----  -------

EOF

    @top =
          (
           sort { $a->[1] <=> $b->[1] }
           map  { [ $_, ($self->{posters}{$_}->orig_volume
                         / $self->{posters}{$_}->body_volume) ] }
           grep { $self->{posters}{$_}->articles >= 3 }
           keys %{$self->{posters}}
          )[0 .. 9];

    for (reverse @top) {
        my $poster = $self->{posters}{$_->[0]};

        printf "%.4f   %5.1f / %5.1f  %5d  %s\n",
               $_->[1], $poster->orig_volume / 1024,
               $poster->body_volume / 1024,
               $poster->articles, $poster->attrib;
    }

    print <<EOF;

Top 10 Threads by Number of Posts
=================================

Posts  Subject
-----  -------

EOF

    @top =
          (
           map  { $_->[0] }
           sort { $b->[1] <=> $a->[1] }
           map  { [ $_, $self->{threads}{$_}->articles ] }
           keys %{$self->{threads}}
          );

    for (@top[0 .. 9]) {
        my $thr = $self->{threads}{$_};
        my $in_thr = $thr->articles;

        printf "%5d  %s\n", $in_thr, $thr->subject;
    }

    print <<EOF;

Top 10 Thread types by Number of Posts
======================================

Posts  Thread-type
-----  -----------

EOF

    for (@top) {
        my $thr = $self->{threads}{$_};
        my $in_thr = $thr->articles;

		  my $thr_type = 'NONE';
		  my $foo = (split(/\s+/,$thr->subject))[0];
		  if ($foo !~ /[a-z]/ || $foo =~ /[>:\]]$/) {$thr_type = $foo;}
		  $thr_type =~ tr/a-z/A-Z/;
		  $thr_type =~ s/[:<>\[\]]*//g;

		  $thr_cnt{$thr_type} += $in_thr;
    }
	 my ($i,$thr_type);
	 foreach $thr_type (sort byvalue (keys %thr_cnt)) {
		  printf "%5d  %s\n", $thr_cnt{$thr_type}, $thr_type;
		  $i++;
		  last if $i>9;
	 }
	 sub byvalue { $thr_cnt{$b} <=> $thr_cnt{$a}; }

    print <<EOF;

Top 10 Threads by Volume
========================

  (kb)   (kb)  (kb)  (kb)
Volume (  hdr/ body/ orig)  Posts  Subject
--------------------------  -----  -------

EOF

    @top =
          (
           map  { $_->[0] }
           sort { $b->[1] <=> $a->[1] }
           map  { [ $_, $self->{threads}{$_}->volume ] }
           keys %{$self->{threads}}
          )[0 .. 9];

    for (@top) {
        my $thr = $self->{threads}{$_};
        my $vol = sprintf "%5.1f (%5.1f/%5.1f/%5.1f)",
                          $thr->volume / 1024,
                          $thr->header_volume / 1024,
                          $thr->body_volume / 1024,
                          $thr->orig_volume / 1024;

        printf "%26s  %5d  %s\n", $vol, $thr->articles,
               $thr->subject;
    }

}

1;
